//
//  resummationSCET.hh
//  ResHiggs
//
//  Created by Luca Rottoli on 29/10/2014.
//


#pragma once

#include "resummation.hh"

using namespace std;

namespace resumm
{
    
/**
  * Class SCETResummation
  */
class SCETResummation: public Resummation
{
  
private:
  
  bool fPi;

  double epsilon;

  double   fmus, fmutmuhrat, fmuHreal;
  dcomplex fmuh, fmut;
  bool fmusset, fmuHset;

  dcomplex fasmuh, fasmut;
  double   fasmus, fasmuf, fasmur;
    
  double fGammaA0, fGammaA1, fGammaA2, fGammaA3;
  double fGammaS0, fGammaS1, fGammaS2;
  double fGammaT0, fGammaT1, fGammaT2;
  double fGammaB0, fGammaB1, fGammaB2;
  double fbbeta0, fbbeta1, fbbeta2, fbbeta3;
    
  dcomplex faGA0, faGA1, faGA2, faGA3;
  dcomplex faGS0, faGS1, faGS2, faGS3;
  dcomplex faGB0, faGB1, faGB2, faGB3;
  dcomplex faGT0, faGT1, faGT2, faGT3;
    
  void ComputeCommonSCET(bool);

  void InitAlphas();
  void Eta();
  void H();
  void U();
  void SoftFunction(dcomplex N);
  void SoftFunction_LargeN(dcomplex N);
  
  double DeltaKFactor(double);
  
  double StildeFactor(double mus);
  
  double fEta, fEta1, fEta2, fEta3, fEta4;
  double fH0, fH1, fH2, fH3;
  double fULL, fUNLL, fUNNLL, fUNNNLL;
  dcomplex fS0, fS1, fS2, fS3;
  dcomplex fSoft0, fSoft1, fSoft2, fSoft3;

  bool fSub;
  bool fStrictExpansion;

  void InitResummCoeffsSCET();
  void InitCoeffHiggsSCET();
  
  dcomplex StildeOneLoop(dcomplex N);
  dcomplex StildeOneLoop_LargeN(dcomplex N);
  
  dcomplex lum_times_Coeff_Function(dcomplex N);
  dcomplex lum_times_Stilde(dcomplex N);

  dcomplex Kres(dcomplex N,bool);
  dcomplex Ksub(dcomplex N,bool);
  dcomplex K(dcomplex N,bool);

  double SCETMinimalPrescription();
  double StildeMinimalPrescription();

  double lalphas(double);
  
  friend double SCETMinimal_integrand(double t, void *p);
  friend double SCETSubtraction_integrand(double t, void *p);
  
  friend double StildeMinimal_integrand(double t, void *p);
  
  
public:
  
  //Set Methods
  void SetmuH(double mu) {fmuHreal = mu; fmuHset=true; };
  void Setmus(double mu) {fmus = mu; fmusset = true; };
  void Resetmus() {fmusset = false; };
  void SetmutmuhRatio(double rat) {fmutmuhrat = rat; };
  void SetStrictExpansion(bool strict) {fStrictExpansion = strict;}
  void SetEpsilon(double ep) {epsilon=ep; };

  //Get Methods
  double GetmusI (double m, double sqrts);
  double GetmusII(double m, double sqrts);
  inline double Getmuh() {return abs(fmuh);};
  
  //Pi Resummation
  void UsePi2Resummation(const bool pi=true)  { fPi = pi; };
  
  SCETResummation(process p, collider c, string PDFset, int PDFmember=0): Resummation(p, c, PDFset, PDFmember), fPi(false), epsilon(1), fmutmuhrat(1), fmusset(false), fmuHset(false), fSub(false), fStrictExpansion(false) { SetPrescription(BP2); SetAPimprovement(AP0); }
  ~SCETResummation();

  double xsSCETLL   (double m, double sqrt_s, double muF_m_ratio=1., double muR_m_ratio=1., double muH_m_ratio=1., order match_order=LO  , double *asinput=NULL);
  double xsSCETNLL  (double m, double sqrt_s, double muF_m_ratio=1., double muR_m_ratio=1., double muH_m_ratio=1., order match_order=NLO , double *asinput=NULL);
  double xsSCETNNLL (double m, double sqrt_s, double muF_m_ratio=1., double muR_m_ratio=1., double muH_m_ratio=1., order match_order=NNLO, double *asinput=NULL);
  double xsSCETNNNLL(double m, double sqrt_s, double muF_m_ratio=1., double muR_m_ratio=1., double muH_m_ratio=1., order match_order=NNLO, double *asinput=NULL);

  double res_xs (double m, double sqrt_s, double muF_m_ratio, double muR_m_ratio, log_order LOGo, order match_order, double *asinput=NULL);
  double sigma_contribution(double m, double sqrt_s, double muF_m_ratio, double muH_m_ratio, log_order LOGo, order match_order, double mus, double *asinput=NULL);
  double res_xs_rap(double m, double Y, double sqrt_s, double muF_m_ratio, double muH_m_ratio, log_order LOGo, order match_order, double *asinput=NULL);
  
  };

};
